# CloudHub(Android)集成文档

## 一、准备

在您阅读此文档时，我们假定您已经具备了基础的Android应用开发经验，并能够理解相关基础概念。

## 二、说明

适用市面主流android 6.0及以上版本的移动终端设备。

## 三、集成SDK

已AndroidStudio为例

### 1. 导入aar包到自己项目libs下

**aar包说明**

| aar               | 说明           |
| ----------------- | -------------- |
| resources.aar     | 需要的资源文件 |
| cloudhub_rtc.aar  | webrtc相关     |
| signalsdk.aar     | 信令消息相关   |
| liveroomsdk.aar   | 教室相关       |
| whiteboardsdk.aar | 白板sdk相关    |
| Whiteboardui.aar  | 白板ui相关     |
| wangxiaosdk.aar   | 网校相关       |

### 2. 在build.gradle中依赖aar包

1. 添加repositories/flatDir/dirs
2. 在dependencies中implementation aar

```gradle
repositories {
    flatDir {
        dirs 'libs'
    }
}

dependencies {
    implementation fileTree(dir: 'libs', include: ['*.jar'])

    implementation(name: 'resources', ext: 'aar')
    implementation(name: 'cloudhub_rtc', ext: 'aar')
    implementation(name: 'signalsdk', ext: 'aar')
    implementation(name: 'liveroomsdk', ext: 'aar')
    implementation(name: 'whiteboardsdk', ext: 'aar')
    implementation(name: 'whiteboardui', ext: 'aar')
    implementation(name: 'wangxiaosdk', ext: 'aar')
}
```

### 3. 处理ndk相关

1. 在导入aar包的module里，找到`build.gradle`，在 `android / defaultConfig` 节点下，添加下面的代码

```gradle
ndk {
    abiFilters "armeabi-v7a", "x86", "x86_64", "arm64-v8a"
}
```

2. 在导入aar包的module里，找到`build.gradle`，在 `android` 节点下，添加如下代码

```gradle
packagingOptions {
    pickFirst 'lib/arm64-v8a/libc++_shared.so'
    pickFirst 'lib/armeabi-v7a/libc++_shared.so'
    pickFirst 'lib/x86/libc++_shared.so'
    pickFirst 'lib/x86_64/libc++_shared.so'
}
```

### 4. 添加其他依赖项

```gradle
// android support
implementation 'com.android.support:recyclerview-v7:28.0.0'
implementation 'com.android.support:design:28.0.0'
implementation 'com.android.support:appcompat-v7:28+'

// 换肤
implementation 'skin.support:skin-support:3.1.4'

// x5 webview
implementation 'com.tencent.tbs.tbssdk:sdk:43903'

// glide
implementation('com.github.bumptech.glide:glide:4.9.0') {
    exclude group: "com.android.support"
}
annotationProcessor 'com.github.bumptech.glide:compiler:4.9.0'

// 适配刘海屏水滴屏等全面屏
implementation 'com.github.smarxpan:NotchScreenTool:0.0.1'

// pdf
implementation 'com.github.barteksc:pdfium-android:1.9.0'

// bugly
implementation 'com.tencent.bugly:crashreport:3.3.3'
implementation 'com.tencent.bugly:nativecrashreport:3.7.7'

// 网络相关
implementation 'com.loopj.android:android-async-http:1.4.9'
implementation 'cz.msebera.android:httpclient:4.4.1.2'
implementation("com.squareup.okhttp3:okhttp:4.9.0")

// 拼音
implementation 'com.belerweb:pinyin4j:2.5.0'
```

### 5. 添加权限

```xml
<!-- Android8.0 允许安装未知来源权限 -->
<uses-permission android:name="android.permission.REQUEST_INSTALL_PACKAGES"/>

<!-- 访问电话状态（处理电话广播.） -->
<uses-permission android:name="android.permission.READ_PHONE_STATE"/>

<!-- 允许程序调用killBackgroundProcesses(String).方法结束后台进程-->
<uses-permission android:name="android.permission.KILL_BACKGROUND_PROCESSES" />

 <!-- 允许访问摄像头进行拍照 -->
<uses-permission android:name="android.permission.CAMERA" />

<!-- 允许程序录制音频 -->
<uses-permission android:name="android.permission.RECORD_AUDIO" />

<!-- 修改声音设置信息 -->
<uses-permission android:name="android.permission.MODIFY_AUDIO_SETTINGS" />

<!-- 允许程序访问网络连接，可能产生GPRS流量 -->
<uses-permission android:name="android.permission.INTERNET" />

<!-- 获取网络信息状态，如当前的网络连接是否有效 -->
<uses-permission android:name="android.permission.ACCESS_NETWORK_STATE" />

<!-- 允许程序访问Wi-Fi网络状态信息 -->
<uses-permission android:name="android.permission.ACCESS_WIFI_STATE"/>

<!-- 允许使用PowerManager的 WakeLocks保持进程在休眠时从屏幕消失 -->
<uses-permission android:name="android.permission.WAKE_LOCK" />

<!-- 允许程序读取底层系统日志文件 -->
<uses-permission
    android:name="android.permission.READ_LOGS"
    tools:ignore="ProtectedPermissions" />

<!-- sd卡文件夹创建和删除 -->
<uses-permission android:name="android.permission.WRITE_EXTERNAL_STORAGE" />
<uses-permission android:name="android.permission.READ_EXTERNAL_STORAGE" />
<uses-permission
    android:name="android.permission.MOUNT_UNMOUNT_FILESYSTEMS"
    tools:ignore="ProtectedPermissions" />

<!--允许程序在手机后后台进程仍然运行-->
<uses-permission android:name="android.permission.WAKE_LOCK" />
<!--显示系统窗口-->
<uses-permission android:name="android.permission.SYSTEM_ALERT_WINDOW" />
```

## 四、集成后的 build.gradle 示例

```gradle
apply plugin: 'com.android.application'
apply plugin: 'bugly'

android {
    
    compileSdkVersion 28
    buildToolsVersion '28.0.3'
    defaultConfig {
        applicationId "com.cloudhub"
        minSdkVersion 19
        targetSdkVersion 28
        versionCode 84
        versionName "3.5.5"
        multiDexEnabled true
        testInstrumentationRunner "android.support.test.runner.AndroidJUnitRunner"
        ndk {
            abiFilters "armeabi-v7a", "x86", "x86_64", "arm64-v8a"
        }
        buildConfigField("String", "buildTime", "\"" + new Date().format("yyyy-MM-dd HH:mm") + "\"")
    }
    buildTypes {
        release {
            minifyEnabled true
            zipAlignEnabled true
            shrinkResources true
            proguardFiles getDefaultProguardFile('proguard-android-optimize.txt'), 'proguard-rules.pro'
        }
        debug {
            shrinkResources false
            minifyEnabled false
        }
    }

    lintOptions {
        abortOnError false
        disable 'MissingTranslation'
    }

    compileOptions {
        sourceCompatibility JavaVersion.VERSION_1_8
        targetCompatibility JavaVersion.VERSION_1_8
    }

    packagingOptions {
        pickFirst 'lib/arm64-v8a/libc++_shared.so'
        pickFirst 'lib/armeabi-v7a/libc++_shared.so'
        pickFirst 'lib/x86/libc++_shared.so'
        pickFirst 'lib/x86_64/libc++_shared.so'
    }
}

repositories {
    flatDir {
        dirs 'libs'
    }
}

dependencies {
    implementation fileTree(dir: 'libs', include: ['*.jar'])
    // test
    testImplementation 'junit:junit:4.12'
    androidTestImplementation 'com.android.support.test:runner:1.0.2'
    androidTestImplementation 'com.android.support.test.espresso:espresso-core:3.0.2'

    implementation(name: 'resources', ext: 'aar')
    implementation(name: 'cloudhub_rtc', ext: 'aar')
    implementation(name: 'signalsdk', ext: 'aar')
    implementation(name: 'liveroomsdk', ext: 'aar')
    implementation(name: 'whiteboardsdk', ext: 'aar')
    implementation(name: 'whiteboardui', ext: 'aar')
    implementation(name: 'wangxiaosdk', ext: 'aar')

    // android support
    implementation 'com.android.support:recyclerview-v7:28.0.0'
    implementation 'com.android.support:design:28.0.0'
    implementation 'com.android.support:appcompat-v7:28+'

    // 换肤
    implementation 'skin.support:skin-support:3.1.4'

    // x5 webview
    implementation 'com.tencent.tbs.tbssdk:sdk:43903'

    // glide
    implementation('com.github.bumptech.glide:glide:4.9.0') {
        exclude group: "com.android.support"
    }
    annotationProcessor 'com.github.bumptech.glide:compiler:4.9.0'

    // 适配刘海屏水滴屏等全面屏
    implementation 'com.github.smarxpan:NotchScreenTool:0.0.1'

    // pdf
    implementation 'com.github.barteksc:pdfium-android:1.9.0'

    // bugly
    implementation 'com.tencent.bugly:crashreport:3.3.3'
    implementation 'com.tencent.bugly:nativecrashreport:3.7.7'

    // 网络相关
    implementation 'com.loopj.android:android-async-http:1.4.9'
    implementation 'cz.msebera.android:httpclient:4.4.1.2'
    implementation("com.squareup.okhttp3:okhttp:4.9.0")

    // 拼音
    implementation 'com.belerweb:pinyin4j:2.5.0'
}
```

## 五、初始化项目

在项目 `Application` 的 `onCreate()` 中，添加如下代码

```java
// bugly收集崩溃日志
CrashReport.initCrashReport(getApplicationContext(), "Your Bugly APPID", false);

// 搜集本地tbs内核信息并上报服务器，服务器返回结果决定使用哪个内核。
QbSdk.PreInitCallback cb = new QbSdk.PreInitCallback() {
    @Override
    public void onViewInitFinished(boolean arg0) {
        // x5內核初始化完成的回调，为true表示x5内核加载成功，否则表示x5内核加载失败，会自动切换到系统内核。
        Log.e("cloudhub", " onViewInitFinished is " + arg0);
    }

    @Override
    public void onCoreInitFinished() {

    }
};

//x5内核初始化接口
QbSdk.initX5Environment(getApplicationContext(), cb);
QbSdk.setDownloadWithoutWifi(true);

// 换肤
SkinCompatManager.withoutActivity(this)
        .addStrategy(new CHSDCardLoader())
        .setSkinStatusBarColorEnable(false)
        .setSkinWindowBackgroundEnable(false)
        .loadSkin();
```

**Android28以上版本，应用无法再让多个进程共用同一 WebView 数据，添加以下代码修复此问题**

> 参考：[Android P 行为变更对 WebView 的影响](https://www.sunzn.com/2019/04/18/Android-P-%E8%A1%8C%E4%B8%BA%E5%8F%98%E6%9B%B4%E5%AF%B9-WebView-%E7%9A%84%E5%BD%B1%E5%93%8D/)

```java
@Override
protected void attachBaseContext(Context base) {
    super.attachBaseContext(base);
    initWebView();
}

private void initWebView() {
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
        String processName = getProcessName(this);
        if (!PROCESS.equals(processName)) {
            WebView.setDataDirectorySuffix(TextUtils.isEmpty(processName) ? "ch" : processName);
        }
    }
}

private String getProcessName(Context context) {
    if (context == null) {
        return null;
    }
    ActivityManager manager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
    for (ActivityManager.RunningAppProcessInfo processInfo : manager.getRunningAppProcesses()) {
        if (processInfo.pid == android.os.Process.myPid()) {
            return processInfo.processName;
        }
    }
    return null;
}
```

## 六、加入教室

### 1. joinRoom

`public void joinRoom(Activity activity, Map<String,Object> map)`

| 参数     | 描述                                                         |
| -------- | ------------------------------------------------------------ |
| activity | 当前Activity                                                 |
| map      | 进入教室参数集合 <br/><br/>params.put("host", "api.263cv.net");//服务器地址<br/>params.put("port", 443);//请求端口号 http--80 https--443<br/>params.put("serial", "serial");//课堂号<br/>params.put("password", ""); //进入课堂人员身份密码（例如：老师。学生）<br/>params.put("userrole", 2);//身份 0老师 1助教 2 学生  3旁听 4隐身用户<br/>params.put("nickname", name);//昵称 （字符串）<br/>params.put("userid", userid);//自定义ID，可不传，如需自己管理，可添加自定义ID（字符串，且只能含数字和字母，不能含其他特殊字符） |

### 2. 回调接口说明

1. RoomNotify

| 方法 | 说明 |
| -- | -- |
| onKickOut | 被请出教室<br/><br/>res == RoomVariable.Kickout_Repeat 有相同身份的人进入教室<br/>res == RoomVariable.Kickout_ChairmanKickout 被老师请出教室 |
| onClassBegin | 上课<br/><br/>type == 3 课堂<br/>type == 4 直播 |
| onClassDismiss | 下课 <br/><br/>type == 3 课堂<br/>type == 4 直播 |

2. JoinRoomCallBack

| 方法 | 说明 |
| -- | -- |
| code	| 描述 |
| 4001	| 公司不存在 |
| 4007	| 教室不存在 |
| -1	| authkey或企业id都为空 |
| 4109	| Auth不正确 |
| 3001	| 数据库配置问题 |
| 3002	| 企业状态异常 |
| 4112	| 教室点数超限 |
| 3003	| 教室过期或删除 |
| 4111	| 非注册用户，该教室要求必须注册用户 |
| 4008	| 密码错误 |
| 4110	| 教室密码不能为空 |
| 4012	| 密码输入错误 |
| 0	| 成功 |

### 3. setDocParams

设置H5课件需要的参数

`public void setDocParams(Map<String, String> docParams)`

## 七、其他说明

1. 注意添加权限和注册 `JoinRoomCallBack`
2. 详情查看 CloudHub SDK demo
3. 为兼容Android9.0，务必在 `application` 节点添加 `android:usesCleartextTraffic="true"`
4. 注意 `Application` 在 `AndroidManifest` 中引用