package com.cloudhub;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.BitmapRegionDecoder;
import android.graphics.Color;
import android.graphics.Rect;
import android.graphics.drawable.BitmapDrawable;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.text.TextUtils;
import android.util.DisplayMetrics;
import android.view.View;
import android.webkit.WebView;
import android.widget.ImageView;
import android.widget.LinearLayout;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.bumptech.glide.request.RequestOptions;
import com.bumptech.glide.request.target.SimpleTarget;
import com.bumptech.glide.request.transition.Transition;
import com.liveroomsdk.base.BaseActivity;
import com.liveroomsdk.view.barview.TitleBar;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * <p>
 * 协议界面
 */
public class AgreementActivity extends BaseActivity {

    private TitleBar mTitleBar;
    private LinearLayout mLlImg;
    private WebView webView;
    private Rect mRect = new Rect();
    private String mTitle;
    private String mUrl;
    private int mScreenWidth;
    private double mScale;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_agreement);

        mTitle = getIntent().getStringExtra("title");
        mUrl = getIntent().getStringExtra("url");
        initView();
        initData();
    }

    private void initView() {
        setTitleBar();
        mTitleBar = findViewById(R.id.title_bar);
        mLlImg = findViewById(R.id.ll_img);
        webView = findViewById(R.id.webView);
        webView.loadUrl(mUrl);
        mTitleBar.setTitle(mTitle);
        mTitleBar.setImmersive(true);
        mTitleBar.setBackgroundColor(getResources().getColor(com.liveroomsdk.R.color.actionbar_bg));
        mTitleBar.setTitleColor(Color.WHITE);
        mTitleBar.setLeftImageResource(com.liveroomsdk.R.mipmap.icon_activity_back);

        mTitleBar.setLeftClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });
    }

    private void initData() {
        DisplayMetrics dm = new DisplayMetrics();
        AgreementActivity.this.getWindowManager().getDefaultDisplay().getMetrics(dm);
        mScreenWidth = dm.widthPixels;

        loadBigImage();
    }

    private void loadBigImage() {
        if (TextUtils.isEmpty(mUrl)) {
            return;
        }
        Glide.with(AgreementActivity.this)
                .asBitmap()
                .apply(new RequestOptions().diskCacheStrategy(DiskCacheStrategy.NONE))
                .load(mUrl)
                .into(new SimpleTarget<Bitmap>() {
                    @Override
                    public void onResourceReady(@NonNull Bitmap resource, @Nullable Transition<? super Bitmap> transition) {
                        setBitmapToImg(resource);
                    }
                });
    }

    private void setBitmapToImg(Bitmap resource) {
        try {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            if (mUrl.endsWith("jpg")) {
                resource.compress(Bitmap.CompressFormat.JPEG, 80, baos);
            } else {
                resource.compress(Bitmap.CompressFormat.PNG, 80, baos);
            }

            InputStream isBm = new ByteArrayInputStream(baos.toByteArray());

            //BitmapRegionDecoder newInstance(InputStream is, boolean isShareable)
            //用于创建BitmapRegionDecoder，isBm表示输入流，只有jpeg和png图片才支持这种方式，
            // isShareable如果为true，那BitmapRegionDecoder会对输入流保持一个表面的引用，
            // 如果为false，那么它将会创建一个输入流的复制，并且一直使用它。即使为true，程序也有可能会创建一个输入流的深度复制。
            // 如果图片是逐步解码的，那么为true会降低图片的解码速度。如果路径下的图片不是支持的格式，那就会抛出异常
            BitmapRegionDecoder decoder = BitmapRegionDecoder.newInstance(isBm, true);

            final int imgWidth = decoder.getWidth();
            final int imgHeight = decoder.getHeight();

            // 屏幕宽度与图片实际宽度的比例
            mScale = (double) mScreenWidth / imgWidth;

            BitmapFactory.Options opts = new BitmapFactory.Options();

            //计算图片要被切分成几个整块，
            // 如果sum=0 说明图片的长度不足3000px，不进行切分 直接添加
            // 如果sum>0 先添加整图，再添加多余的部分，否则多余的部分不足3000时底部会有空白
            int sum = imgHeight / 3000;

            int redundant = imgHeight % 3000;

            //说明图片的长度 < 3000
            if (sum == 0) {
                //直接加载
                addChildImg(resource);
            } else {
                //说明需要切分图片
                for (int i = 0; i < sum; i++) {
                    //需要注意：mRect.set(left, top, right, bottom)的第四个参数，
                    //也就是图片的高不能大于这里的4096
                    mRect.set(0, i * 3000, imgWidth, (i + 1) * 3000);
                    Bitmap bm = decoder.decodeRegion(mRect, opts);
                    addChildImg(bm);
                }

                //将多余的不足3000的部分作为尾部拼接
                if (redundant > 0) {
                    mRect.set(0, sum * 3000, imgWidth, imgHeight);
                    Bitmap bm = decoder.decodeRegion(mRect, opts);
                    addChildImg(bm);
                }
            }

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 裁剪bitmap，添加到linearlayout
     *
     * @param bm bitmap
     */
    private void addChildImg(Bitmap bm) {
        ImageView imageView = new ImageView(AgreementActivity.this);
        imageView.setImageBitmap(bm);
        mLlImg.addView(imageView, mScreenWidth, (int) (bm.getHeight() * mScale));
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (mLlImg != null) {
            for (int i = 0; i < mLlImg.getChildCount(); i++) {
                ImageView view = (ImageView) mLlImg.getChildAt(i);
                view.setImageBitmap(null);

                Bitmap bitmap = ((BitmapDrawable) view.getDrawable()).getBitmap();
                if (bitmap != null && !bitmap.isRecycled()) {
                    bitmap.recycle();
                    bitmap = null;
                }

                System.gc();
            }
            mLlImg.removeAllViews();
            mLlImg = null;
        }
    }
}
