package com.cloudhub;

import android.content.Context;
import android.graphics.Color;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.content.ContextCompat;
import android.support.v7.widget.AppCompatTextView;
import android.text.SpannableString;
import android.text.Spanned;
import android.text.TextPaint;
import android.text.method.LinkMovementMethod;
import android.text.style.ClickableSpan;
import android.util.AttributeSet;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.View;

import com.resources.R;

import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class AgreementView extends AppCompatTextView {

    private Context mContext;

    public AgreementView(Context context) {
        this(context, null);
    }

    public AgreementView(Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public AgreementView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        mContext = context;
        init();
        initAgreement();
    }

    private void init() {
        this.setGravity(Gravity.CENTER);
        this.setLineSpacing((int) mContext.getResources().getDimension(R.dimen.dp_3), 1F);
        this.setMinHeight((int) mContext.getResources().getDimension(R.dimen.dp_30));
        this.setText(mContext.getResources().getString(R.string.login_agreement));
        this.setTextColor(ContextCompat.getColor(mContext, R.color.ch_color_gray));
        this.setTextSize(TypedValue.COMPLEX_UNIT_PX, mContext.getResources().getDimension(R.dimen.sp_10));
    }

    private void initAgreement() {
        String agreement = this.getText().toString();

        ArrayList<Integer> start = new ArrayList<>();
        ArrayList<Integer> end = new ArrayList<>();

        Pattern p = Pattern.compile("《(.*?)》");
        Matcher m = p.matcher(agreement);
        while (m.find()) {
            start.add(m.start());
            end.add(m.end());
        }

        if (start.size() < 2 || end.size() < 2) {
            return;
        }

        SpannableString s = new SpannableString(agreement);
        s.setSpan(new ClickableSpan() {
            @Override
            public void updateDrawState(@NonNull TextPaint ds) {
                super.updateDrawState(ds);
                ds.setUnderlineText(true);
                ds.setColor(mContext.getResources().getColor(R.color.ch_bg_color));
            }

            @Override
            public void onClick(@NonNull View widget) {
                LoginTool.startAgreementPrivacy(mContext);
            }
        }, start.get(0), end.get(0), Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
        s.setSpan(new ClickableSpan() {
            @Override
            public void updateDrawState(@NonNull TextPaint ds) {
                super.updateDrawState(ds);
                ds.setUnderlineText(true);
                ds.setColor(mContext.getResources().getColor(R.color.ch_bg_color));
            }

            @Override
            public void onClick(@NonNull View widget) {
                LoginTool.startAgreementUser(mContext);
            }
        }, start.get(1), end.get(1), Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
        this.setHighlightColor(Color.TRANSPARENT);
        this.setMovementMethod(LinkMovementMethod.getInstance());
        this.setText(s);
    }
}
