package com.cloudhub;

import android.annotation.SuppressLint;
import android.app.ActivityManager;
import android.app.Application;
import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.os.Build;
import android.os.Debug;
import android.text.TextUtils;
import android.webkit.WebView;

import com.cloudhub.whiteboardsdk.manage.CloudHubWhiteBoardKit;
import com.liveroomsdk.common.BuildVars;
import com.resources.theme.SDCardLoader;
import com.tencent.bugly.crashreport.CrashReport;
import com.tencent.bugly.crashreport.crash.h5.H5JavaScriptInterface;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Locale;

import skin.support.SkinCompatManager;


public class ApplicationCH extends Application {

    private static final String PROCESS = "com.cloudhub";

    @Override
    public void onCreate() {
        super.onCreate();

//        if (BuildConfig.DEBUG) {
//            StrictMode.setThreadPolicy(new StrictMode.ThreadPolicy.Builder()
//                    .detectDiskReads()
//                    .detectDiskWrites()
//                    .detectNetwork()
//                    .penaltyLog()
//                    .build());
//            StrictMode.setVmPolicy(new StrictMode.VmPolicy.Builder()
//                    .detectLeakedSqlLiteObjects()
//                    .detectLeakedClosableObjects()
//                    .penaltyLog()
//                    .penaltyDeath()
//                    .build());
//        }

        CrashReport.initCrashReport(getApplicationContext(), BuildVars.BUGLY_APP_ID, false);
        CrashReport.WebViewInterface webView = new CrashReport.WebViewInterface() {
            @Override
            public String getUrl() {
                return null;
            }

            @Override
            public void setJavaScriptEnabled(boolean b) {

            }

            @Override
            public void loadUrl(String s) {

            }

            @Override
            public void addJavascriptInterface(H5JavaScriptInterface h5JavaScriptInterface, String s) {

            }

            @Override
            public CharSequence getContentDescription() {
                return null;
            }
        };
        webView.setJavaScriptEnabled(false);
        CrashReport.setJavascriptMonitor(webView, true);

        //初始化
        CloudHubWhiteBoardKit.getInstance().initWhiteBoardSDK(getApplicationContext());
        // 换肤
        SkinCompatManager.withoutActivity(this)
                .addStrategy(new SDCardLoader())
                .setSkinStatusBarColorEnable(false)
                .setSkinWindowBackgroundEnable(false)
                .loadSkin();

        // debug模式下，关掉Android9的弹窗
        if (BuildConfig.DEBUG && Build.VERSION.SDK_INT >= 28) {
            closeAndroidPDialog();
        }

        checkDynamicInjectionattackRisk();
    }

    @Override
    protected void attachBaseContext(Context base) {
        super.attachBaseContext(base);
        initWebView();
    }

    private void initWebView() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
            String processName = getProcessName(this);
            if (!PROCESS.equals(processName)) {
                WebView.setDataDirectorySuffix(TextUtils.isEmpty(processName) ? "ch" : processName);
            }
        }
    }

    private String getProcessName(Context context) {
        if (context == null)
            return null;
        ActivityManager manager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        for (ActivityManager.RunningAppProcessInfo processInfo : manager.getRunningAppProcesses()) {
            if (processInfo.pid == android.os.Process.myPid()) {
                return processInfo.processName;
            }
        }
        return null;
    }

    /**
     * 关闭弹窗
     */
    private void closeAndroidPDialog() {
        try {
            Class packClss = Class.forName("android.content.pm.PackageParser$Package");
            Constructor declaredConstructor = packClss.getDeclaredConstructor(String.class);
            declaredConstructor.setAccessible(true);
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            Class ThreadCls = Class.forName("android.app.ActivityThread");
            Method declaredMethod = ThreadCls.getDeclaredMethod("currentActivityThread");
            declaredMethod.setAccessible(true);
            Object activityThread = declaredMethod.invoke(null);
            @SuppressLint("SoonBlockedPrivateApi")
            Field mHiddenApiWarningShown = ThreadCls.getDeclaredField("mHiddenApiWarningShown");
            mHiddenApiWarningShown.setAccessible(true);
            mHiddenApiWarningShown.setBoolean(activityThread, true);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /***
     *  防动态注入攻击风险
     */
    private void checkDynamicInjectionattackRisk() {
        if (!BuildConfig.DEBUG) {
            if (isDebuggable()) {
                System.exit(0);
            }
            Thread t = new Thread(new Runnable() {
                @Override
                public void run() {
                    while (true) {
                        try {
                            Thread.sleep(300);
                            if (Debug.isDebuggerConnected()) {
                                System.exit(0);
                            }

                            if (isUnderTraced()) {
                                System.exit(0);
                            }
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    }
                }

            }, "SafeGuardThread");
            t.start();
        }
        if (isUnderTraced()) {
            System.exit(0);
        }
    }

    private boolean isUnderTraced() {
        String processStatusFilePath = String.format(Locale.US, "/proc/%d/status", android.os.Process.myPid());
        File procInfoFile = new File(processStatusFilePath);
        try {
            BufferedReader b = new BufferedReader(new FileReader(procInfoFile));
            String readLine;
            while ((readLine = b.readLine()) != null) {
                if (readLine.contains("TracerPid")) {
                    String[] arrays = readLine.split(":");
                    if (arrays.length == 2) {
                        int tracerPid = Integer.parseInt(arrays[1].trim());
                        if (tracerPid != 0) {
                            return true;
                        }
                    }
                }
            }

            b.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    public boolean isDebuggable() {
        return 0 != (getApplicationInfo().flags & ApplicationInfo.FLAG_DEBUGGABLE);
    }
}
