package com.cloudhub;

import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.support.v4.content.FileProvider;
import android.text.TextUtils;
import android.view.KeyEvent;

import com.cloudhub.whiteboardsdk.net.OkHttpUtil;
import com.liveroomsdk.common.BuildVars;
import com.resources.manage.BaseDialog;
import com.resources.utils.Tools;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import okhttp3.Call;
import okhttp3.Response;


public class AutoUpdateUtil {

    private static AutoUpdateUtil mInstance = null;
    private BaseDialog mDownloadDialog;
    private String mTarget;
    private Activity mActivity;
    private BaseDialog mCHDialog = null;
    // APK下载地址
    private String mApkDownLoadUrl = "";

    private DialogInterface.OnKeyListener keylistener = new DialogInterface.OnKeyListener() {
        @Override
        public boolean onKey(DialogInterface dialog, int keyCode, KeyEvent event) {
            if (keyCode == KeyEvent.KEYCODE_BACK && event.getRepeatCount() == 0) {
                return true;
            } else {
                return false;
            }
        }
    };

    static public AutoUpdateUtil getInstance() {
        synchronized ("") {
            if (mInstance == null) {
                mInstance = new AutoUpdateUtil();
            }
            return mInstance;
        }
    }

    public void resetInstance() {
        if (mDownloadDialog != null) {
            mDownloadDialog = null;
        }
        if (!TextUtils.isEmpty(mTarget)) {
            mTarget = null;
        }
        if (mCHDialog != null) {
            mCHDialog = null;
        }
        mInstance = null;
    }

    //检测自动更新
    public void checkForUpdates(Activity activity) {
        mActivity = activity;
        String url = BuildVars.AUTO_UPDATE_HEADER + BuildVars.getWebServer() + ":" +
                443 + "/ClientAPI/checkupdateinfo";
        Map<String, String> bodyMap = new HashMap<>();
        bodyMap.put("type", "2");
        bodyMap.put("version", BuildVars.versions);
        bodyMap.put("level", "4");
        bodyMap.put("ts", LoginTool.dateToStamp());
        /*params.put("companydomain","1209gwcs");*/

        OkHttpUtil.getInstance().postFormAsyncData(url, bodyMap, new OkHttpUtil.ResponseCallBack() {

            @Override
            public void onResponse(okhttp3.Call call, Response response) {
                try {
                    JSONObject jsonObject = Tools.toJson(response);
                    if (jsonObject != null) {
                        final int nRet = jsonObject.getInt("result");
                        if (nRet == 0) {
                            mApkDownLoadUrl = Tools.optString(jsonObject, "updateaddr");
                            //updateflag 升级标志 0:不升級   1：强制升级  1：不强制升级
                            final int code = jsonObject.optInt("updateflag");
                            mActivity.runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    if (code != 0) {
                                        checkForUpdataDialog(code);
                                    }
                                }
                            });
                        }
                    }

                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onFailure(okhttp3.Call call, IOException e) {

            }
        });
    }

    // 制定更新提示框
    public void checkForUpdataDialog(int code) {
        String msg = "";

        if (mCHDialog == null) {
            mCHDialog = new AutoupdateDialog(mActivity);
        }
        if (code == 1) {
            msg = mActivity.getString(R.string.find_new_version_fouce);
            ((AutoupdateDialog) mCHDialog).setdissVisibility();
        } else if (code == 2) {
            msg = mActivity.getString(R.string.find_new_version);
        }
        mCHDialog.setMessage(msg);
        mCHDialog.setOnKeyListener(keylistener);
        mCHDialog.canceledOnTouchOutside(false);
        mCHDialog.setListener(new BaseDialog.OnCHDialogListener() {
            @Override
            public void onSureClick(String data) {
                downLoadApp();
            }
        });
        if (mCHDialog != null && !mCHDialog.isShowing() && !mActivity.isFinishing()) {
            mCHDialog.show();
        }
    }

    /***
     * 下载APK
     */
    public void downLoadApp() {
        mTarget = Environment.getExternalStorageDirectory().getAbsolutePath() + "/CloudHub.apk";
        if (!(mApkDownLoadUrl.contains("http://") || mApkDownLoadUrl.contains("https://"))) {
            mApkDownLoadUrl = BuildVars.AUTO_UPDATE_HEADER + mApkDownLoadUrl;
        }
        if (!mApkDownLoadUrl.isEmpty()) {
            File localFile = new File(mTarget);
            if (!localFile.exists()) {//创建文件夹
                localFile.getParentFile().mkdirs();
            }
            OkHttpUtil.getInstance().getAsyncDownload(mApkDownLoadUrl, localFile, new OkHttpUtil.DownLoadFileCallBack() {

                @Override
                public void onFailure(Call call, File file, IOException e) {
                    ((Activity) mActivity).runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            if (mDownloadDialog != null && mActivity != null &&
                                    !mActivity.isFinishing() && !mActivity.isDestroyed()) {
                                mDownloadDialog.dismiss();
                            }
                        }
                    });
                }

                @Override
                public void onSuccess(Call call, File file) {
                    ((Activity) mActivity).runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            if (mDownloadDialog != null && mActivity != null &&
                                    !mActivity.isFinishing() && !mActivity.isDestroyed()) {
                                mDownloadDialog.dismiss();
                            }
                            // 安装apk
                            installApk();
                        }
                    });
                }

                @Override
                public void onProgress(long bytesWritten, long totalSize) {
                    ((Activity) mActivity).runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            setCurrentValue(totalSize, bytesWritten);
                        }
                    });
                }
            });
        }
    }

    /**
     * 设置当前进度值
     */
    public void setCurrentValue(long maxvalue, long current) {
        if (mDownloadDialog == null) {
            mDownloadDialog = new DownloadDialog(mActivity);
            mDownloadDialog.canceledOnTouchOutside(false);
        }
        if (mActivity != null && !mActivity.isFinishing()) {
            if (mDownloadDialog.isShowing()) {
                ((DownloadDialog) mDownloadDialog).updateProgress(maxvalue, current);
            } else {
                mDownloadDialog.show();
            }
        }
    }

    //安装APK
    protected void installApk() {
        if (TextUtils.isEmpty(mTarget)) {
            return;
        }
        Intent intent = new Intent();
        intent.setAction("android.intent.action.VIEW");
        if (Build.VERSION.SDK_INT >= 24) {//判读版本是否在7.0以上){
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            File file = new File(mTarget);
            Uri apkUri = FileProvider.getUriForFile(mActivity, mActivity.getPackageName() + ".fileprovider", file);//在AndroidManifest中的android:authorities值
            intent.setDataAndType(apkUri, "application/vnd.android.package-archive");
            mActivity.startActivity(intent);
        } else {
            Uri data = Uri.parse("file://" + mTarget);
            intent.setDataAndType(data, "application/vnd.android.package-archive");
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            mActivity.startActivity(intent);
        }
    }
}
