package com.cloudhub;

import android.graphics.Color;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.SeekBar;
import android.widget.TextView;

import com.liveroomsdk.base.BaseActivity;
import com.liveroomsdk.view.barview.TitleBar;
import com.resources.utils.SPUtils;

/**
 * 护眼模式
 */
public class EyeprotectionActivity extends BaseActivity {

    private TitleBar mTitleBar;
    private SwitchButton mSwitchOpenEye;
    private SwitchButton mSwitchNoRemind;
    private SeekBar mSeekbar;
    private TextView mTvseekbarNum;
    private Button mButok;

    //是否开启护眼模式
    private boolean isEyeOpenOrClose;
    //是否保存提醒
    private boolean isSaveRemind;
    //屏幕亮度
    private int mSaveScreen;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_eyeprotection);
        initView();
        initClick();
        initData();
    }

    private void initView() {
        setTitleBar();
        mTitleBar = findViewById(R.id.title_bar);
        mSwitchOpenEye = findViewById(R.id.sw_open_eye);
        mSwitchNoRemind = findViewById(R.id.sw_no_remind);
        mSeekbar = findViewById(R.id.seek_bar);
        mTvseekbarNum = findViewById(R.id.tv_seekbar_num);
        mButok = findViewById(R.id.but_ok);

        mTitleBar.setTitle(getResources().getString(R.string.eys_setting));
        mTitleBar.setImmersive(true);
        mTitleBar.setBackgroundColor(getResources().getColor(com.liveroomsdk.R.color.actionbar_bg));
        mTitleBar.setTitleColor(Color.WHITE);
        mTitleBar.setLeftImageResource(com.liveroomsdk.R.mipmap.icon_activity_back);

        mSeekbar.setMax(100);
    }

    private void initClick() {
        mTitleBar.setLeftClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });
        //开启护眼功能
        mSwitchOpenEye.setOnCheckedChangeListener(new SwitchButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(SwitchButton view, boolean isChecked) {
                if (isChecked) {
                    mWindowView.setBackgroundColor(Color.parseColor("#1AFFECB2"));
                } else {
                    mWindowView.setBackgroundColor(Color.TRANSPARENT);
                }
                isEyeOpenOrClose = isChecked;
            }
        });
        //保存提醒
        mSwitchNoRemind.setOnCheckedChangeListener(new SwitchButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(SwitchButton view, boolean isChecked) {
                isSaveRemind = isChecked;
            }
        });
        //拖动
        mSeekbar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                mSaveScreen = progress;
                mTvseekbarNum.setText(getString(R.string.screen_num) + ":" + progress);
                setBrightness(EyeprotectionActivity.this, progress);
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

        // 确认保存
        mButok.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                //保存护眼
                SPUtils.put(EyeprotectionActivity.this, "EyeData", "EyeOpenOrClose", isEyeOpenOrClose);
                //保存提醒
                SPUtils.put(EyeprotectionActivity.this, "EyeData", "SaveRemind", isSaveRemind);
                //屏幕亮度
                SPUtils.put(EyeprotectionActivity.this, "EyeData", "SaveScreen", mSaveScreen);
                finish();
            }
        });
    }

    private void initData() {
        isEyeOpenOrClose = (boolean) SPUtils.get(this, "EyeData", "EyeOpenOrClose", false);
        isSaveRemind = (boolean) SPUtils.get(this, "EyeData", "SaveRemind", false);
        mSaveScreen = (int) SPUtils.get(this, "EyeData", "SaveScreen", getScreenBrightness(this));

        mSwitchOpenEye.setChecked(isEyeOpenOrClose);
        mSwitchNoRemind.setChecked(isSaveRemind);
        mSeekbar.setProgress(mSaveScreen);
        mTvseekbarNum.setText(getString(R.string.screen_num) + ":" + mSaveScreen);
    }
}

