package com.cloudhub;

import android.app.ActivityManager;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.RequiresApi;
import android.text.Editable;
import android.text.InputType;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.cloudhub.whiteboardsdk.net.OkHttpUtil;
import com.liveroomsdk.base.BaseActivity;
import com.liveroomsdk.common.BuildVars;
import com.liveroomsdk.common.RoomVariable;
import com.liveroomsdk.dialog.CHDialogOfLogin;
import com.liveroomsdk.dialog.CHInputPwdDialog;
import com.liveroomsdk.interfaces.JoinRoomCallBack;
import com.liveroomsdk.interfaces.RoomNotify;
import com.liveroomsdk.manage.RoomClient;
import com.liveroomsdk.manage.RoomSession;
import com.liveroomsdk.utils.PermissionUtil;
import com.liveroomsdk.view.CHLoading;
import com.resources.manage.BaseDialog;
import com.resources.theme.ThemeManager;
import com.resources.utils.SPUtils;
import com.resources.utils.Tools;
import com.resources.utils.toast.ToastUtils;
import com.smarx.notchlib.NotchScreenManager;
import com.wangxiaosdk.Config;
import com.wangxiaosdk.LoginReady;
import com.wangxiaosdk.activity.WangxiaoActivity;
import com.wangxiaosdk.utils.RsaEncryptUtil;
import com.whiteboardui.utils.KeyboardUtils;
import com.whiteboardui.utils.NetUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import cloudhub.signal.room.RoomInterface;
import okhttp3.Call;
import okhttp3.Response;

public class LoginActivity extends BaseActivity implements JoinRoomCallBack,
        RoomNotify, TextWatcher, View.OnClickListener {

    private EditText mName;
    private EditText mAccout;
    private EditText mOrg;
    private ImageView mLogo;

    private TextView mVersion;
    private TextView mWxSwitch;
    private Button mLogin;
    private ImageView mSetting;
    // loading
    private CHLoading mLoadingView;

    // 当前是否是网校登陆
    private boolean isWangxiao;
    private HashMap<String, Object> mParams = new HashMap<>();

    private String mPublicKey;

    @RequiresApi(api = Build.VERSION_CODES.P)
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if (RoomSession.isInRoom || (getIntent().getFlags() &
                Intent.FLAG_ACTIVITY_BROUGHT_TO_FRONT) != 0) {
            finish();
            return;
        }

        NotchScreenManager.getInstance().setDisplayInNotch(this);
        setContentView(R.layout.activity_login);

        mAccout = findViewById(R.id.accout);
        mOrg = findViewById(R.id.wx_org);
        mLogo = findViewById(R.id.logo);
        mName = findViewById(R.id.name);
        mLogin = findViewById(R.id.login);
        mWxSwitch = findViewById(R.id.wx_switch);
        mVersion = findViewById(R.id.tv_version);
        mLoadingView = findViewById(R.id.loading);
        mSetting = findViewById(R.id.im_setting);

        mSetting.setOnClickListener(this);
        mLogin.setOnClickListener(this);
        mWxSwitch.setOnClickListener(this);
        mName.addTextChangedListener(this);
        mOrg.addTextChangedListener(this);

        handleIntent(getIntent());
        //自动更新检测
        AutoUpdateUtil.getInstance().checkForUpdates(this);
        //权限检查
        PermissionUtil.requestPermission(this);
        //注册房间回调
        RoomClient.getInstance().registerInterface(this, this);

        //检测时间 晚22：00 - 6：00
        if (LoginTool.isCurrentInTimeScope(22, 00, 06, 00)) {
            boolean isSaveRemind = (boolean) SPUtils.get(this, "EyeData",
                    "SaveRemind", false);
            //是否开启不再提醒
            if (!isSaveRemind) {
                showErrorDialog("", 4);
            }
        }

        initLoginInfo();
        LoginReady.getmInstance().regiestInterface(this, this);

        StringBuilder version = new StringBuilder();
        version.append(LoginTool.packageName(this));
        if (BuildConfig.DEBUG) {
            version.append("-");
            version.append(BuildConfig.buildTime);
        }
        mVersion.setText(String.format("V%s", version));
    }

    @Override
    protected void onStop() {
        super.onStop();
        mParams.clear();
        LoginReady.getmInstance().resetVariables();
        updateLoginState(true);
        // 如果是从scheme打开的，会有2个LoginActivity，关闭当前的
        if (getIntent().getFlags() == 0) {
            finish();
        }
        mLoadingView.hide();
    }

    @Override
    protected void onResume() {
        super.onResume();
        ThemeManager.getInstance().resetTheme();
    }

    /**
     * 获取保存的登录信息
     */
    private void initLoginInfo() {
        String roomNumber = (String) SPUtils.get(this, "RoomNuberAndNick", "RoomNumber", "");
        String nick = (String) SPUtils.get(this, "RoomNuberAndNick", "Nick", "");

        mAccout.setText(roomNumber);
        mAccout.setSelection(mAccout.getText().length());

        mName.setText(nick);
        mName.setSelection(mName.getText().length());
    }

    /**
     * 获取保存的网校登录信息
     */
    private void initWxLoginInfo() {
        String domain = (String) SPUtils.get(this, "RoomNuberAndNick", "WX_Domain", "");
        String account = (String) SPUtils.get(this, "RoomNuberAndNick", "WX_Account", "");
        String pwd = (String) SPUtils.get(this, "RoomNuberAndNick", "WX_Pwd", "");

        mOrg.setText(domain);
        mOrg.setSelection(mOrg.getText().length());

        mAccout.setText(account);
        mAccout.setSelection(mAccout.getText().length());

        mName.setText(pwd);
        mName.setSelection(mName.getText().length());
    }

    /**
     * 登录检测
     */
    private void login() {
        String name = mName.getText().toString().trim();
        String accout = mAccout.getText().toString().trim();
        if (TextUtils.isEmpty(accout)) {
            updateLoginState(true);
            ToastUtils.show(this, getString(R.string.account_not_exist));
            return;
        }
        if (TextUtils.isEmpty(name)) {
            updateLoginState(true);
            ToastUtils.show(this, getString(R.string.name_not_exist));
            return;
        }

        LoginTool.saveLoginInfo(this, accout, name);
        mParams.clear();

        mParams.put("serial", accout);//课堂号
        mParams.put("nickname", name);
        mParams.put("port", 443);
        mParams.put("host", BuildVars.getWebServer());

        if (NetUtils.isNetAvailable(this)) {
            updateLoginState(true);
            ToastUtils.show(this, getString(R.string.unable_connect_network));
            return;
        }

        LoginReady.getmInstance().getRoomType(accout, mParams);
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        handleIntent(intent);
    }

    /**
     * @param code 错误码
     */
    @Override
    public void callBack(int code) {
        updateLoginState(true);
        KeyboardUtils.hideInputMethod(this);

        switch (code) {
            case 0:
                break;

            case 4001:
                showErrorDialog(getString(R.string.error_4001), 0);
                break;
            case 4007:
                showErrorDialog(getString(R.string.error_4007), 0);
                break;
            case -1:
                showErrorDialog(getString(R.string.error__1), 0);
                break;
            case 4109:
                showErrorDialog(getString(R.string.error_4109), 0);
                break;
            case 3001:
                showErrorDialog(getString(R.string.error_3001), 0);
                break;
            case 3002:
                showErrorDialog(getString(R.string.error_3002), 0);
                break;
            case 4112:
                showErrorDialog(getString(R.string.error_4112), 0);
                break;
            case 3003:
                showErrorDialog(getString(R.string.error_3003), 0);
                break;
            case 4111:
                showErrorDialog(getString(R.string.error_4111), 0);
                break;
            case 4008:
                showErrorDialog(getString(R.string.error_4008), 0);
                break;
            case 4110:
                showErrorDialog(getString(R.string.error_4110), 3);
                break;
            case 4012:
                showErrorDialog(getString(R.string.error_4012), 0);
                break;
            case 900:
                showErrorDialog(getString(R.string.error_900), 0);
                break;
            case 4999:
            case 104:
                showErrorDialog(getString(R.string.error_104), 0);
                break;


            //checkroomtype为407 三次失敗响应码（2001为android端自定义）
            case 2001:
                showErrorDialog(getString(R.string.roomtype_3), 0);
                break;
            //checkroomtype响应码为4007（2002为android端自定义）
            case 2002:
                showErrorDialog(getString(R.string.roomtype_4007), 0);
                break;
            //需要选择进入身份--学生需要密码（2003为android端自定义)
            case 2003:
                showErrorDialog("", 2);
                break;
            //需要选择进入身份--学不生需要密码（2004为android端自定义)
            case 2004:
                showErrorDialog("", 5);
                break;
            default:
                break;
        }
    }

    /***
     * 更新Login的状态
     */
    private void updateLoginState(boolean isClickable) {
        if (mLogin == null || mLoadingView == null) {
            return;
        }
        if (isClickable) {
            mLogin.setClickable(true);
            mLogin.setAlpha(1F);
            mLoadingView.hide();
        } else {
            mLoadingView.show();
            mLogin.setClickable(false);
            mLogin.setAlpha(0.4F);
        }
    }

    /**
     * 显示对话框
     */
    private void showErrorDialog(String msg, int type) {
        BaseDialog ysDialog = null;
        if (type == 1 || type == 3) {
            ysDialog = new CHInputPwdDialog(this);
            if (type == 3) {
                ((CHInputPwdDialog) ysDialog).setViewState(3);
            }
            ysDialog.setListener(new BaseDialog.OnCHDialogListener() {
                @Override
                public void onSureClick(String data) {
                    mLoadingView.show();
                    try {
                        JSONObject jsonObject = new JSONObject(data);
                        mParams.put("password", Tools.optString(jsonObject, "password"));
                        RoomClient.getInstance().joinRoom(LoginActivity.this, mParams);
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                }
            });
        } else if (type == 0) {
            ysDialog = new CHDialogOfLogin(this)
                    .setMessage(msg);
        } else if (type == 2 || type == 5) {
            ysDialog = new CHInputPwdDialog(this);
            if (type == 5) {
                ((CHInputPwdDialog) ysDialog).setViewState(5);
            }
            ysDialog.setListener(new BaseDialog.OnCHDialogListener() {
                @Override
                public void onSureClick(String data) {
                    mLoadingView.show();
                    try {
                        JSONObject jsonObject = new JSONObject(data);
                        String role = Tools.optString(jsonObject, "userrole");
                        String grouproom = LoginReady.getmInstance().getGrouproom();
                        if ((grouproom.equals("1") || grouproom.equals("2")) && role.equals("0")) {
                            callBack(900);
                            return;
                        }
                        if (grouproom.equals("1") && (role.equals("2") || role.equals("4"))) {
                            callBack(900);
                            return;
                        }
                        mParams.put("password", jsonObject.optString("password"));
                        mParams.put("userrole", role);
                        RoomClient.getInstance().joinRoom(LoginActivity.this, mParams);
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                }
            });
        } else if (type == 4) {  //护眼设置
            ysDialog = new EyeCareDialog(this);
            ysDialog.canceledOnTouchOutside(false);
            ysDialog.setListener(new BaseDialog.OnCHDialogListener() {
                @Override
                public void onSureClick(String data) {
                    startActivity(new Intent(LoginActivity.this, EyeprotectionActivity.class));
                }
            });
        }
        if (ysDialog != null && !ysDialog.isShowing() && !this.isFinishing()) {
            ysDialog.show();
        }
    }

    @Override
    public void onKickOut(int res) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (res == RoomVariable.KickoutChairmanKickout) {
                    ToastUtils.show(LoginActivity.this, getString(R.string.kick_out_tip), Toast.LENGTH_LONG);
                }
                if (res == RoomVariable.KickoutRepeat) {
                    ToastUtils.show(LoginActivity.this, getString(R.string.chairman_kick_out), Toast.LENGTH_LONG);
                }
            }
        });
    }

    @Override
    public void onClassBegin(int type) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                String msg = null;
                if (type == 3) {
                    msg = getString(R.string.class_started);
                } else if (type == 4) {
                    msg = getString(R.string.live_started);
                }
                if (!TextUtils.isEmpty(msg)) {
                    ToastUtils.show(LoginActivity.this, msg, Toast.LENGTH_LONG);
                }
            }
        });
    }

    @Override
    public void onClassDismiss(int type) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                String msg = null;
                if (type == 3) {
                    msg = getString(R.string.class_closeed);
                } else if (type == 4) {
                    msg = getString(R.string.live_closeed);
                }
                if (!TextUtils.isEmpty(msg)) {
                    ToastUtils.show(LoginActivity.this, msg, Toast.LENGTH_LONG);
                }
            }
        });
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        ActivityManager am = (ActivityManager) getSystemService(Context.ACTIVITY_SERVICE);
        am.killBackgroundProcesses(getPackageName());
        System.exit(0);

        RoomInterface.getInstance().destroy();
        // 如果不是从scheme打开的，重置状态
        if (getIntent().getFlags() != 0) {
            RoomClient.getInstance().resetInstance();
        }
        AutoUpdateUtil.getInstance().resetInstance();
        LoginReady.getmInstance().resetInstance();
    }

    private void handleIntent(Intent intent) {
        if (intent != null) {
            Uri uri = intent.getData();
            if (uri != null) {
                String url = uri.toString();
                if (url.startsWith("enterlivev3://")) {
                    String temp = url.substring(url.indexOf("?") + 1);
                    temp = Uri.decode(temp);
                    String[] temps = temp.split("&");
                    Map<String, Object> map = new HashMap<>();
                    for (String s : temps) {
                        String[] t = s.split("=");
                        if (t.length > 1) {
                            map.put(t[0], t[1]);
                        }
                    }
                    if (map.containsKey("logintype")) {
                        Object type = map.get("logintype");
                        if (type instanceof String) {
                            int logintype = Integer.parseInt((String) type);
                            if (logintype == 1) {
                                return;
                            }
                        }
                    }
                    if (map.containsKey("roomid")) {
                        String roomid = (String) map.get("roomid");
                        if (mAccout != null) {
                            mAccout.setText(roomid);
                        }
                    } else {
                        if (mLoadingView != null) {
                            mLoadingView.show();
                        }
                        RoomClient.getInstance().joinRoom(LoginActivity.this, map);
                    }
                }
            }
        }
    }

    @Override
    public void beforeTextChanged(CharSequence s, int start, int count, int after) {

    }

    @Override
    public void onTextChanged(CharSequence s, int start, int before, int count) {
        if (isWangxiao) {
            String string = mOrg.getText().toString();
            if (string.contains(" ")) {
                String replaceAll = string.replaceAll(" ", "");
                mOrg.setText(replaceAll);
                Tools.editTextSelectionEnd(mOrg);
            }
        } else {
            String name = mName.getText().toString();
            int maxLength = 20;
            if (LoginTool.isContainChinese(name)) {
                maxLength = 10;
            }
            if (name.length() > maxLength) {
                ToastUtils.show(this, getString(R.string.login_input_limit));
                mName.setText(s.toString().substring(0, maxLength));
                Tools.editTextSelectionEnd(mName);
            }
        }
    }

    @Override
    public void afterTextChanged(Editable s) {

    }

    @Override
    public void onClick(View view) {
        int id = view.getId();
        switch (id) {
            case R.id.login:
                if (isWangxiao) {
                    checkWangxiao();
                    return;
                }
                updateLoginState(false);
                login();
                break;

            case R.id.im_setting:
                startActivity(new Intent(this, SettingActivity.class));
                break;

            case R.id.wx_switch:
                isWangxiao = !isWangxiao;
                switchLogin();
                break;

            default:
                break;
        }
    }

    /**
     * 切换登陆（网校 ｜ 正常）
     */
    private void switchLogin() {
        if (isWangxiao) {
            LoginReady.getmInstance().unRegiestInterface();
            RoomClient.getInstance().unRegiestInterface();
            initWxLoginInfo();
            mLogo.setImageResource(R.mipmap.icon_logo_wx_ch);
            RelativeLayout.LayoutParams params = (RelativeLayout.LayoutParams) mLogo.getLayoutParams();
            params.bottomMargin = (int) this.getResources().getDimension(R.dimen.dp_20);
            mLogo.setLayoutParams(params);
            mOrg.setVisibility(View.VISIBLE);
            mAccout.setInputType(InputType.TYPE_CLASS_TEXT);
            mAccout.setHint(this.getResources().getString(R.string.login_wx_account_hint));
            mName.setHint(this.getResources().getString(R.string.login_wx_pwd_hint));
            mName.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_PASSWORD);
            mLogin.setText(this.getResources().getString(R.string.login_enter));
            mWxSwitch.setText(this.getResources().getString(R.string.login_text));
        } else {
            LoginReady.getmInstance().regiestInterface(this, this);
            RoomClient.getInstance().registerInterface(this, this);
            initLoginInfo();
            mLogo.setImageResource(R.mipmap.icon_class_ch);
            RelativeLayout.LayoutParams params = (RelativeLayout.LayoutParams) mLogo.getLayoutParams();
            params.bottomMargin = (int) this.getResources().getDimension(R.dimen.dp_20);
            mLogo.setLayoutParams(params);
            mOrg.setVisibility(View.GONE);
            mAccout.setInputType(InputType.TYPE_CLASS_NUMBER);
            mAccout.setHint(this.getResources().getString(R.string.login_accout_hint));
            mName.setHint(this.getResources().getString(R.string.login_name_hint));
            mName.setInputType(InputType.TYPE_CLASS_TEXT);
            mLogin.setText(this.getResources().getString(R.string.login_text));
            mWxSwitch.setText(this.getResources().getString(R.string.login_wx));
        }
    }

    /**
     * 获取publicKey
     */
    private void getPublicKey(final String domain, final String admin_account, final String admin_pwd) {
        if (NetUtils.isNetAvailable(this)) {
            updateLoginState(true);
            ToastUtils.show(this, getString(R.string.unable_connect_network));
            return;
        }
        OkHttpUtil.getInstance().postFormAsyncData(Config.getPublickeyUrl(this), null, new OkHttpUtil.ResponseCallBack() {
            @Override
            public void onResponse(Call call, Response response) {

                JSONObject jsonObject = Tools.toJson(response);
                if (jsonObject != null) {
                    mPublicKey = jsonObject.optString("key");
                    if (TextUtils.isEmpty(mPublicKey)) {
                        runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                ToastUtils.show(LoginActivity.this, "public key is empty");
                                updateLoginState(true);
                            }
                        });
                        return;
                    }
                    loginWangxiao(domain, admin_account, admin_pwd);
                }
            }

            @Override
            public void onFailure(Call call, IOException e) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        if (!TextUtils.isEmpty(e.getMessage()) && e.getMessage().startsWith("UnknownHostException")) {
                            ToastUtils.show(LoginActivity.this, getString(R.string.login_company_error));
                        } else {
                            ToastUtils.show(LoginActivity.this, "get Public key error: " + e.getMessage());
                        }
                        updateLoginState(true);
                    }
                });
            }
        });
    }

    /**
     * 登陆网校
     */
    private void checkWangxiao() {

        String domain = mOrg.getText().toString().trim().replaceAll(" ", "");
        String admin_account = mAccout.getText().toString().trim();
        String admin_pwd = mName.getText().toString().trim();

        if (!checkParams(domain, admin_account, admin_pwd)) {
            return;
        }

        Config.setHost(this, domain);

        updateLoginState(false);

        LoginTool.saveWXLoginInfo(this, domain, admin_account, admin_pwd);

        getPublicKey(domain, admin_account, admin_pwd);
    }

    private void loginWangxiao(String domain, String admin_account, String admin_pwd) {
        if (NetUtils.isNetAvailable(this)) {
            updateLoginState(true);
            ToastUtils.show(this, getString(R.string.unable_connect_network));
            return;
        }
        String key = LoginTool.getRandomString(10);
        Config.setYsKey(this, key);

        try {
            HashMap<String, Object> map = new HashMap<>();
            map.put("domain", domain);
            map.put("admin_account", admin_account);
            map.put("admin_pwd", admin_pwd);
            map.put("key", key);

            String login_data = RsaEncryptUtil.encryptByPublicKey(new JSONObject(map).toString(), mPublicKey);

            Map<String, String> bodyMap = new HashMap<>();
            bodyMap.put("login_data", login_data);

            Map<String, String> headerMap = new HashMap<>();
            headerMap.put("sign", "");
            headerMap.put("starttime", String.valueOf(System.currentTimeMillis()));
            headerMap.put("token", "");
            OkHttpUtil.getInstance().postFormAsyncData(Config.loginUrl(LoginActivity.this), bodyMap, headerMap, new OkHttpUtil.ResponseCallBack() {
                @Override
                public void onResponse(Call call, Response response) {
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            JSONObject jsonObject = Tools.toJson(response);
                            updateLoginState(true);
                            if (jsonObject == null) {
                                ToastUtils.show(LoginActivity.this, "login: " + " no response");
                                return;
                            }
                            int code = jsonObject.optInt("code");
                            String info = jsonObject.optString("info");
                            // 10213 -- 老师返回  10221 -- 学生返回
                            if (code == 10221 || code == 10213) {
                                ToastUtils.show(LoginActivity.this, LoginActivity.this.getString(R.string.pwd_error));
                                return;
                            }
                            if (code != 0) {
                                ToastUtils.show(LoginActivity.this, "login: " + code + " _ " + info);
                                return;
                            }
                            JSONObject data = jsonObject.optJSONObject("data");
                            if (data == null) {
                                ToastUtils.show(LoginActivity.this, "login: " + " no data");
                                return;
                            }
                            int uid = data.optInt("uid");
                            String token = data.optString("token");
                            String organid = data.optString("organid");
                            int usertype = data.optInt("usertype");
                            String nickname = data.optString("nickname");
                            String mobile = data.optString("mobile");
                            String imageurl = data.optString("imageurl");
                            Config.setUid(LoginActivity.this, uid);
                            Config.setToken(LoginActivity.this, token);
                            Config.setOrganid(LoginActivity.this, organid);
                            Config.setUsertype(LoginActivity.this, usertype);
                            Config.setUsername(LoginActivity.this, nickname);
                            Config.setMobile(LoginActivity.this, mobile);
                            Config.setImageurl(LoginActivity.this, imageurl);
                            startActivity(new Intent(LoginActivity.this, WangxiaoActivity.class));
                        }
                    });
                }

                @Override
                public void onFailure(Call call, IOException e) {
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            ToastUtils.show(LoginActivity.this, "login error: " + e.getMessage());
                            updateLoginState(true);
                        }
                    });
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 检测参数
     *
     * @param domain        域名
     * @param admin_account 账号
     * @param admin_pwd     密码
     */
    private boolean checkParams(String domain, String admin_account, String admin_pwd) {
        if (TextUtils.isEmpty(domain)) {
            ToastUtils.show(this, getString(R.string.login_company_empty));
            return false;
        }

        if (domain.contains(".") && !Tools.isDomain(domain)) {
            ToastUtils.show(this, getString(R.string.login_company_error));
            return false;
        }

        if (TextUtils.isEmpty(admin_account)) {
            ToastUtils.show(this, getString(R.string.login_accout_empty));
            return false;
        }

        if (TextUtils.isEmpty(admin_pwd)) {
            ToastUtils.show(this, getString(R.string.login_pwd_empty));
            return false;
        }
        return true;
    }
}
