package com.cloudhub;

import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.text.format.Time;

import com.liveroomsdk.common.BuildVars;
import com.resources.utils.SPUtils;

import java.util.Date;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class LoginTool {

    /**
     * 保存登录信息
     *
     * @param accout 教室号
     * @param name   用户名
     */
    public static void saveLoginInfo(Context context, String accout, String name) {
        SPUtils.put(context, "RoomNuberAndNick", "RoomNumber", accout);
        SPUtils.put(context, "RoomNuberAndNick", "Nick", name);
    }


    /**
     * 保存网校登录信息
     */
    public static void saveWXLoginInfo(Context context, String domain, String account, String pwd) {
        SPUtils.put(context, "RoomNuberAndNick", "WX_Domain", domain);
        SPUtils.put(context, "RoomNuberAndNick", "WX_Account", account);
        SPUtils.put(context, "RoomNuberAndNick", "WX_Pwd", pwd);
    }

    /**
     * 获取应用的版本号versionCode：
     *
     * @param context context
     * @return code
     */
    public static int packageCode(Context context) {
        PackageManager manager = context.getPackageManager();
        int code = 0;
        try {
            PackageInfo info = manager.getPackageInfo(context.getPackageName(), 0);
            code = info.versionCode;
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        return code;
    }

    /**
     * 获取应用的版本信息
     *
     * @param context context
     * @return 版本号
     */
    public static String packageName(Context context) {
        PackageManager manager = context.getPackageManager();
        String name = null;
        try {
            PackageInfo info = manager.getPackageInfo(context.getPackageName(), 0);
            name = info.versionName;
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        return name;
    }

    /**
     * 判断字符串是否包含中文
     */
    public static boolean isContainChinese(String str) {
        Pattern p = Pattern.compile("[\u4e00-\u9fa5]");
        Matcher m = p.matcher(str);
        return m.find();
    }

    /**
     * 判断当前系统时间是否在特定时间的段内
     *
     * @param beginHour 开始的小时，例如5
     * @param beginMin  开始小时的分钟数，例如00
     * @param endHour   结束小时，例如 8
     * @param endMin    结束小时的分钟数，例如00
     * @return true表示在范围内，否则false
     */
    public static boolean isCurrentInTimeScope(int beginHour, int beginMin, int endHour, int endMin) {
        boolean result = false;// 结果
        final long aDayInMillis = 1000 * 60 * 60 * 24;// 一天的全部毫秒数
        final long currentTimeMillis = System.currentTimeMillis();// 当前时间

        Time now = new Time();// 注意这里导入的时候选择android.text.format.Time类,而不是java.sql.Time类
        now.set(currentTimeMillis);

        Time startTime = new Time();
        startTime.set(currentTimeMillis);
        startTime.hour = beginHour;
        startTime.minute = beginMin;

        Time endTime = new Time();
        endTime.set(currentTimeMillis);
        endTime.hour = endHour;
        endTime.minute = endMin;

        if (!startTime.before(endTime)) {
            // 跨天的特殊情况（比如22:00-8:00）
            startTime.set(startTime.toMillis(true) - aDayInMillis);
            result = !now.before(startTime) && !now.after(endTime); // startTime <= now <= endTime
            Time startTimeInThisDay = new Time();
            startTimeInThisDay.set(startTime.toMillis(true) + aDayInMillis);
            if (!now.before(startTimeInThisDay)) {
                result = true;
            }
        } else {
            // 普通情况(比如 8:00 - 14:00)
            result = !now.before(startTime) && !now.after(endTime); // startTime <= now <= endTime
        }
        return result;
    }

    /**
     * 获取一条随机字符串
     *
     * @param length 字符串长度
     */
    public static String getRandomString(int length) {
        String base = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        Random random = new Random();
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < length; i++) {
            int number = random.nextInt(base.length());
            sb.append(base.charAt(number));
        }
        return sb.toString();
    }

    public static String dateToStamp() {
        String res = null;
        Date date = new Date(System.currentTimeMillis());
        long ts = date.getTime();
        res = String.valueOf(ts);
        return res;
    }

    /**
     * 用户协议
     */
    public static void startAgreementUser(Context context) {
        Intent intent = new Intent(context, AgreementActivity.class);
        intent.putExtra("title", context.getResources().getString(R.string.agreement_user));
        intent.putExtra("url", BuildVars.getYsAgreementUserUrl());
        context.startActivity(intent);
    }

    /**
     * 隐私政策
     */
    public static void startAgreementPrivacy(Context context) {
        Intent intent = new Intent(context, AgreementActivity.class);
        intent.putExtra("title", context.getResources().getString(R.string.agreement_privacy));
        intent.putExtra("url", BuildVars.getYsAgreementPrivacyUrl());
        context.startActivity(intent);
    }
}
