package com.cloudhub;

import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.content.ContextCompat;
import android.view.KeyEvent;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.cloudhub.whiteboardsdk.manage.CloudHubWhiteBoardKit;
import com.cloudhub.whiteboardsdk.model.HostEnums;
import com.cloudhub.whiteboardsdk.widget.X5WebView;
import com.liveroomsdk.base.BaseActivity;
import com.liveroomsdk.common.BuildVars;
import com.liveroomsdk.view.barview.TitleBar;

/**
 * 设置
 */
public class SettingActivity extends BaseActivity implements View.OnClickListener {

    private RelativeLayout mRootView;
    private TextView mAgreementUser;
    private TextView mAgreementPrivacy;
    private X5WebView mX5WebView;
    private TextView mApi;
    private TextView mRelease;
    private TextView mDemo;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_setting);

        mRootView = findViewById(R.id.rel_parent);
        mAgreementUser = findViewById(R.id.agreement_user);
        mAgreementPrivacy = findViewById(R.id.agreement_privacy);
        TitleBar titleBar = findViewById(R.id.title_bar);
        TextView eye = findViewById(R.id.eye);
        TextView debugX5 = findViewById(R.id.debug_webview);
        LinearLayout switchServer = findViewById(R.id.server_switch);
        mApi = findViewById(R.id.api);
        mRelease = findViewById(R.id.release);
        mDemo = findViewById(R.id.demo);

        setTitleBar();
        titleBar.setTitle(getResources().getString(R.string.setting));
        titleBar.setImmersive(true);
        titleBar.setBackgroundColor(getResources().getColor(com.liveroomsdk.R.color.actionbar_bg));
        titleBar.setTitleColor(Color.WHITE);
        titleBar.setLeftImageResource(com.liveroomsdk.R.mipmap.icon_activity_back);
        titleBar.setLeftClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });

        if (BuildConfig.DEBUG) {
            debugX5.setVisibility(View.VISIBLE);
            switchServer.setVisibility(View.VISIBLE);
            updateServer();
        } else {
            debugX5.setVisibility(View.GONE);
            switchServer.setVisibility(View.GONE);
        }

        mAgreementUser.setOnClickListener(this);
        mAgreementPrivacy.setOnClickListener(this);
        eye.setOnClickListener(this);
        debugX5.setOnClickListener(this);
        mApi.setOnClickListener(this);
        mRelease.setOnClickListener(this);
        mDemo.setOnClickListener(this);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (mX5WebView != null) {
            mRootView.removeView(mX5WebView);
            mX5WebView = null;
            return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    /**
     * X5内核调试
     * 注意调试时删除build.gradle中"arm64-v8a"，因为X5不支持64位
     * 打开X5WebView后，选择 【信息】->【TBS settings】，勾选
     * 【是否打开 TBS 内核 Inspector 调试功能】 ，然后app会自动重启
     * 在Chrome中打开chrome://inspect/#devices，即可开始调试
     */
    public void X5Test() {
        if (mX5WebView == null) {
            mX5WebView = new X5WebView(this, null);
            mX5WebView.setBackgroundColor(Color.WHITE);
        }
        mX5WebView.loadUrl("http://debugtbs.qq.com");
        // mX5WebView.loadUrl("http://debugx5.qq.com");
        RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(
                RelativeLayout.LayoutParams.MATCH_PARENT, RelativeLayout.LayoutParams.MATCH_PARENT);
        mX5WebView.setLayoutParams(params);
        mRootView.removeView(mX5WebView);
        mRootView.addView(mX5WebView, params);
    }

    private void updateServer() {
        int buildEnv = BuildVars.getBuildEnv();
        switch (buildEnv) {
            case BuildVars.ENV_DEMO:
                mApi.setTextColor(ContextCompat.getColor(this, R.color.ch_color_gray));
                mRelease.setTextColor(ContextCompat.getColor(this, R.color.ch_color_gray));
                mDemo.setTextColor(ContextCompat.getColor(this, R.color.blue_btn));
                break;
            case BuildVars.ENV_RELEASE:
                mApi.setTextColor(ContextCompat.getColor(this, R.color.ch_color_gray));
                mRelease.setTextColor(ContextCompat.getColor(this, R.color.blue_btn));
                mDemo.setTextColor(ContextCompat.getColor(this, R.color.ch_color_gray));
                break;
            default:
            case BuildVars.ENV_API:
                mApi.setTextColor(ContextCompat.getColor(this, R.color.blue_btn));
                mRelease.setTextColor(ContextCompat.getColor(this, R.color.ch_color_gray));
                mDemo.setTextColor(ContextCompat.getColor(this, R.color.ch_color_gray));
                break;
        }
    }

    @Override
    public void onClick(View v) {
        int id = v.getId();
        if (id == R.id.eye) {
            startActivity(new Intent(this, EyeprotectionActivity.class));
        } else if (id == R.id.agreement_user) {
            LoginTool.startAgreementUser(this);
        } else if (id == R.id.agreement_privacy) {
            LoginTool.startAgreementPrivacy(this);
        } else if (id == R.id.debug_webview) {
            if (BuildConfig.DEBUG) {
                X5Test();
            }
        } else if (id == R.id.api) {
            BuildVars.setBuildEnv(BuildVars.ENV_API);
            CloudHubWhiteBoardKit.getInstance().setHostEnum(HostEnums.API);
            updateServer();
        } else if (id == R.id.release) {
            BuildVars.setBuildEnv(BuildVars.ENV_RELEASE);
            CloudHubWhiteBoardKit.getInstance().setHostEnum(HostEnums.RELEASE);
            updateServer();
        } else if (id == R.id.demo) {
            BuildVars.setBuildEnv(BuildVars.ENV_DEMO);
            CloudHubWhiteBoardKit.getInstance().setHostEnum(HostEnums.DEMO);
            updateServer();
        }
    }
}
